@@ -35,14 +35,16 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
                                    log_trans = TRUE, coe_of_variation=250,
                                    lod_limit = 0, save_plots = TRUE, plot_dir=tempdir()){
   Sample <- cov_tab <- seq_cov <- Dilution <- seq_group <- slope <- intercept <- mag_ab <- NULL
-  seq_det <- grouped_seq_cov <- seq_cov_filt <- lod <- fit <- log_scale <- mag_cov <- NULL
+  seq_det <- grouped_seq_cov <- seq_cov_filt <- lod <- fit <- log_scale <- mag_cov <- zero_row_check <- . <- NULL
+  number_of_groups <- NULL
+  filtered_samples <- character()
+
   # Retrieve sample names from feature tibble
-  scale_fac <- dplyr::tibble(Sample = names(f_tibble) %>%
-                               stringr::str_subset(pattern = "Feature", negate = TRUE))
+  scale_fac <- dplyr::tibble(Sample = names(f_tibble)[-1])
 
   # Merge dilution factors for samples, add log-scaling option
   scale_fac <- scale_fac %>%
-    dplyr::inner_join(seq_dilution, by = "Sample") %>%
+    dplyr::inner_join(seq_dilution %>% stats::setNames(c("Sample", "Dilution")), by = "Sample") %>%
     dplyr::mutate(log_scale = log_trans)
 
   # Make coverage table for features
@@ -88,8 +90,7 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
       seq_det = purrr::map(seq_det, ~ dplyr::mutate(., diff = standards - detected)),
       seq_det = purrr::map(seq_det, ~ dplyr::mutate(., ratio = detected*100/standards)),
       lod = purrr::map_dbl(seq_det, ~ dplyr::filter(., ratio > lod_limit) %>%
-                             dplyr::filter(Concentration == min(Concentration)) %>%
-                             dplyr::pull(Concentration)),
+                             dplyr::pull(Concentration) %>% min()),
       seq_warning = purrr::map_int(seq_det, ~ dplyr::summarise(., Sum = sum(diff)) %>%
                                      dplyr::pull(Sum)) #positive values give warning later
     ) %>%
@@ -121,8 +122,44 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
                                    dplyr::filter(Concentration >= .y) %>%
                                    dplyr::filter(., coe_var <= coe_of_variation) %>% #remove zero coverage values before rlm
                                    dplyr::mutate(
-                                     lod = .y))) %>%
+                                     lod = .y)),
+      zero_row_check = purrr::map(seq_cov_filt, ~nrow(.))
+      )
 
+  scale_fac <- scale_fac %>%
+    dplyr::mutate(
+      number_of_groups = purrr::map_int(scale_fac$grouped_seq_cov, ~ dplyr::filter(.x, coe_var <= coe_of_variation) %>% nrow()),
+      number_of_sequins = purrr::map_int(scale_fac$seq_cov_filt, ~ nrow(.x))
+    )
+
+  if(all(scale_fac$number_of_groups <= 2)) stop("All fractions have 2 or less sequin concentration groups below the coefficient of variation, there is no sufficient number of data to carry out the linear regression, please consider increasing the coefficient value.")
+
+  filtered_samples <- scale_fac %>%
+    dplyr::filter(number_of_groups <= 2) %>%
+    dplyr::pull(Sample) %>%
+    append(filtered_samples, .)
+
+  if(nrow(dplyr::filter(scale_fac, number_of_groups <= 2)) > 0){
+    message(glue::glue("{length(filtered_samples)} fractions were removed because they have 2 or less sequin concentration groups with a coefficient of variation below the coefficient of variation threshold."))
+  }
+
+  scale_fac <- scale_fac %>%
+    dplyr::filter(number_of_groups > 2)
+
+  filtered_samples = scale_fac %>%
+    dplyr::filter(zero_row_check == 0) %>%
+    dplyr::pull(Sample)
+
+  if(nrow(dplyr::filter(scale_fac, zero_row_check <= 1)) > 0){
+    message(glue::glue("{nrow(dplyr::filter(scale_fac, zero_row_check <= 1))} fractions were removed because they have 1 or 0 sequin data points"))
+  }
+
+  scale_fac <- scale_fac %>%
+    dplyr::filter(zero_row_check > 0)
+
+  if(nrow(scale_fac) == 0) stop("There are no fractions with sufficient sequin data points to carry out the linear regression, please consider increasing the coefficient of variation. ")
+
+  scale_fac <- scale_fac %>%
     dplyr::mutate(
       seq_cov_filt = purrr::map(seq_cov_filt, ~ .x %>%
                                   dplyr::mutate(
@@ -134,8 +171,25 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
       ),
       slope = purrr::map_dbl(fit, ~ summary(.)$coef[2]), # get slope
       intercept = purrr::map_dbl(fit, ~summary(.)$coef[1]) # get intercept
-    ) %>%
+    )
 
+  # save samples that have negative slope to raise warning on lost samples
+  filtered_samples <- scale_fac %>%
+    dplyr::filter(slope < 0) %>%
+    dplyr::pull(Sample) %>%
+    append(filtered_samples, .)
+
+  if(nrow(dplyr::filter(scale_fac, slope < 0)) > 0){
+    message(glue::glue("{nrow(dplyr::filter(scale_fac, slope < 0))} fractions were removed because they have a negative regression slope."))
+  }
+
+  # filter samples with negative slope and continue
+  scale_fac <- scale_fac %>%
+    dplyr::filter(slope > 0)
+
+  if(nrow(scale_fac) == 0) stop("There are no fractions that passed the filtering (i.e. they have low sequin data points below the coefficient of variation or they have a negative regression slope), please consider increasing the coefficient of variation.")
+
+  scale_fac <- scale_fac %>%
     #plot linear regressions
     dplyr::mutate(
       plots = ifelse(log_scale == "TRUE" , # check log_trans input
@@ -188,6 +242,9 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
       mag_det = purrr::map2(mag_det, Sample, ~ stats::setNames(.x, c("Feature", .y))) #change header back to sample
     )
 
+  # TODO exit when CoV is too small and there are no sequins
+  # TODO print messsage about how many sequins are being used for the lm fit
+
   # compile feature abundance across samples
   mag_tab <- scale_fac$mag_ab %>%
     purrr::reduce(dplyr::full_join, by="Feature") %>%
@@ -216,12 +273,18 @@ scale_features_rlm <- function(f_tibble, sequin_meta, seq_dilution,
                   "mag_det" = mag_det,
                   "plots" = plots,
                   "scale_fac" = scale_fac)
+  if(length(filtered_samples > 0)){
+    results$filtered_samples = filtered_samples
+    message(glue::glue("{length(filtered_samples)} samples out of {sum(length(filtered_samples),ncol(mag_tab))} were filtered out, see 'filtered_samples' in output list."))
+  }
 
   # return results
   return(results)
 
-  file.remove(plot_dir) #If plots are not saved, this directory will be empty and here it will be removed, if the dir is not empty and a name is not provided the script raises a warning
-
+  # TODO reimplement plot generation
 
+  file.remove(plot_dir) #If plots are not saved, this directory will be empty
+  # and here it will be removed, if the dir is not empty and a name is not
+  # provided the script raises a warning
 }
 
